
public class Chromosome
{
	private String chromosome;
	private static int nGenes;
	private double fitness;
	private double fitnessMin;

	/**
	 * Costruttore di sicurezza in caso venga creato un cromosoma senza specificare 
	 * il parametro del numero dei geni
	 */
	public Chromosome()
	{
		chromosome = "";
		nGenes = 0;
		fitness = 0;
		fitnessMin = 4000;
	}
	
	/**
	 * Costruttore per creare un nuovo oggetto cromosoma con corredo
	 * genetico del tutto random data la lunghezza
	 * @param numberOfGenes
	 */
	public Chromosome(int numberOfGenes)
	{
		nGenes = numberOfGenes;
		chromosome = "";
		for(int i=0; i<nGenes; i++)
		{
			int j = (int) (Math.random()*10) % 3;
			if (j == 0)
				chromosome = chromosome + "A";
			else if (j == 1)
				chromosome = chromosome + "D";
			else
				chromosome = chromosome + "S";				
		}
	}
	
	/**
	 * Costruttore per creare un nuovo oggetto cromosoma 
	 * data la stringa di geni
	 * @param chromosome
	 */
	public Chromosome(String chromosome){
		this.chromosome = chromosome;
	}	

	/**
	 * Restituisce il cromosoma come stringa
	 * @return
	 */
	public String getChromosome()
	{
		return chromosome;
	}	
	public void setChromosome(String c)
	{
		this.chromosome =c;
	}	
	/**
	 * Restituisce il numero di geni dell'individuo
	 * @return
	 */
	public int getGeneNumber()
	{
		return nGenes;
	}		
	
	/**
	 * Restituisce la fitness
	 * @return
	 */
	public double getFitness()
	{
		return fitness;
	}
	public double getFitnessMin()
	{
		return fitnessMin;
	}
//	public double getFitnessMin()
//	{
//		
//		if(fitness==10000||fitness==20000) return 0;
//		else if(fitness==0) return 10000;
//		return 1/fitness;
//	}
	
	/**
	 * Imposta la fitness
	 * @param fitness
	 */
	public void setFitness(double fitness)
	{
		this.fitness = fitness;
	}
	public void setFitnessMin(double fitnessMin)
	{
		this.fitnessMin = fitnessMin;
	}
	
	/**
	 * Sovrascrive il metodo toString
	 */
	public String toString()
	{	
		int n=(int)(fitness*10000);
		double ff=n/10000.0;
		return chromosome+"|FF:"+ff;
	}

	/**
	 * Apporta una mutazione con una certa probabilit a uno dei geni del cromosoma(pu essere lo stesso di quello selezionato)
	 * @param probability
	 */
	public void mutation_simple2(double probability){
		double randomValue=Math.random();
		if(randomValue>probability)
		{
			if(GAMP.systemout) if(GAMP.systemout) System.out.println("Nessuna mutazione");
			return;
		}
		//scelta del gene da modificare
		int indexGene=(int)(Math.random()*(nGenes));	//tolto -1
		if(GAMP.systemout) System.out.println("Mutazione del gene di posizione "+indexGene);
		String head=chromosome.substring(0,indexGene);
		String tail=chromosome.substring(indexGene+1,chromosome.length());
		String gene;
		
		//scelta del nuovo valore da dare al gene mutato
		int j = (int) (Math.random()*10) % 3;
		if (j == 0)
			gene = "A";
		else if (j == 1)
			gene = "D";
		else
			gene = "S";	
		
		if(GAMP.systemout) System.out.println("Scelta del valore nuovo gene: "+gene);
		chromosome=head+gene+tail;
	}	
	/**
	 * Apporta una mutazione con una certa probabilit a uno dei geni del cromosoma cambiando quello selezionato
	 * @param probability
	 */
	public void mutation_simple(double probability){
		double randomValue=Math.random();
		if(randomValue>probability)
		{
			if(GAMP.systemout) System.out.println("Nessuna mutazione");
			return;
		}
		//scelta del gene da modificare
		int indexGene=(int)(Math.random()*(nGenes));	//tolto -1
		if(GAMP.systemout) System.out.println("Mutazione del gene di posizione "+indexGene);
		String head=chromosome.substring(0,indexGene);
		String tail=chromosome.substring(indexGene+1,chromosome.length());
		String gene=chromosome.substring(indexGene,indexGene+1);
		//scelta del nuovo valore da dare al gene mutato in funzione di quello selezionato
		if(gene.equals("A"))
		{
			int j = (int) (Math.random()*10) % 2;
			if (j == 0)
				gene = "S";
			else if (j == 1)
				gene = "D";
		}
		else if(gene.equals("S"))
		{
			int j = (int) (Math.random()*10) % 2;
			if (j == 0)
				gene = "A";
			else if (j == 1)
				gene = "D";
		}
		else if(gene.equals("D"))
		{
			int j = (int) (Math.random()*10) % 2;
			if (j == 0)
				gene = "A";
			else if (j == 1)
				gene = "S";
		}
		if(GAMP.systemout) System.out.println("Scelta del valore nuovo gene: "+gene);
		chromosome=head+gene+tail;
	}	
	/**
	 * Apporta una mutazione con una certa probabilit invertendo due geni del cromosoma
	 * @param probability
	 */
	public void mutation_inverse(double probability){
		double randomValue=Math.random();
		if(randomValue>probability)
		{
			if(GAMP.systemout) System.out.println("Nessuna mutazione");
			return;
		}
		//scelta del gene da modificare
		int indexGene=(int)(Math.random()*(nGenes));
		int indexGene2;
		do
		{
		indexGene2=(int)(Math.random()*(nGenes));//tolto -1
		}
		while(indexGene==indexGene2);//se uguali ricalcolo il secondo
		if(GAMP.systemout) System.out.println("Mutazione per inversione del gene di posizione "+indexGene+" con quello in posizione "+indexGene2);
		if(indexGene>indexGene2)//voglio sempre avere indexGene pi piccolo dell'altro per poter separare la stringa di geni
		{
			int temp=indexGene;
			indexGene=indexGene2;
			indexGene2=temp;
		}
		String head=chromosome.substring(0,indexGene);
		String gene=chromosome.substring(indexGene,indexGene+1);
		String body=chromosome.substring(indexGene+1,indexGene2);
		String gene2=chromosome.substring(indexGene2,indexGene2+1);
		String tail=chromosome.substring(indexGene2+1,chromosome.length());
		chromosome=head+gene2+body+gene+tail;
	}	
	
	/**
	 * Crossover a un punto
	 * @param daddy
	 * @param mummy
	 * @return children (array di due figli)
	 */
	public static Chromosome[] crossoverOnePoint(Chromosome daddy, Chromosome mummy)
	{
		Chromosome[] children = new Chromosome[2];
		
		//Point of Cut
		int poc = (int)(Math.random() * 100) % nGenes;

		//head
		String hDaddy = daddy.getChromosome().substring(0, poc);
		//tail
		String tDaddy = daddy.getChromosome().substring(poc);
		if(GAMP.systemout) System.out.println("Padre: " + hDaddy + "|" + tDaddy);
		String hMummy = mummy.getChromosome().substring(0, poc);
		String tMummy = mummy.getChromosome().substring(poc);
		if(GAMP.systemout) System.out.println("Madre: " + hMummy + "|" + tMummy);
		children[0] = new Chromosome(hDaddy + tMummy);
		children[1] = new Chromosome(hMummy + tDaddy);

		return children;
	}
	
	/**
	 * Crossover a due punti
	 * @param daddy
	 * @param mummy
	 * @return children (array di due figli)
	 */
	public static Chromosome[] crossoverTwoPoints(Chromosome daddy, Chromosome mummy)
	{
		Chromosome[] children = new Chromosome[2];
		
		//Points of Cut 
		int poc1 = (int)(Math.random() * 100) % nGenes;
		int poc2 = (int)(Math.random() * 100) % nGenes;
		//non voglio due punti di taglio uguali 
		//(oppure se vengono fuori 2 punti uguali facciamo fare il crossoverOnePoint?)
		while (poc1 == poc2)
			poc2 = (int)(Math.random() * 100) % nGenes;

		int pocMin = Math.min(poc1, poc2);
		int pocMax = Math.max(poc1, poc2);
		
		//head
		String hDaddy = daddy.getChromosome().substring(0, pocMin);
		//center
		String cDaddy = daddy.getChromosome().substring(pocMin, pocMax);
		//tail
		String tDaddy = daddy.getChromosome().substring(pocMax);
		if(GAMP.systemout) System.out.println("\nCrossover a due punti");
		if(GAMP.systemout) System.out.println("Padre: " + hDaddy + "|" + cDaddy + "|" + tDaddy);
		String hMummy = mummy.getChromosome().substring(0, pocMin);
		String cMummy = mummy.getChromosome().substring(pocMin, pocMax);
		String tMummy = mummy.getChromosome().substring(pocMax);
		if(GAMP.systemout) System.out.println("Madre: " + hMummy + "|" + cMummy + "|" + tMummy);
		children[0] = new Chromosome(hDaddy + cMummy + tDaddy);
		children[1] = new Chromosome(hMummy + cDaddy + tMummy);
		if(GAMP.systemout) System.out.println("Figlio1: " + children[0].getChromosome());
		if(GAMP.systemout) System.out.println("Figlio2: " + children[1].getChromosome());

		return children;
	}
	
	/**
	 * Crossover uniforme (1)
	 * Per ogni gene del primo figlio si scegliere con probabilit 0.5 
	 * qual  il genitore che lo fornisce. Il secondo discendente prender il gene dallaltro genitore.
	 * @param daddy
	 * @param mummy
	 * @return children (array di due figli)
	 */
	public static Chromosome[] uniformCrossover1(Chromosome daddy, Chromosome mummy)
	{
		Chromosome[] children = new Chromosome[2];
		
		String child1 = "";
		String child2 = "";		
		for (int i = 0; i < daddy.getChromosome().length(); i++) 
		{
			double flipCoin = Math.random();
			if (flipCoin <= 0.5)
			{
				child1 = child1 + daddy.getChromosome().charAt(i);
				child2 = child2 + mummy.getChromosome().charAt(i);
			}
			else
			{
				child1 = child1 + mummy.getChromosome().charAt(i);
				child2 = child2 + daddy.getChromosome().charAt(i);
			}			
		}
		
		children[0] = new Chromosome(child1);
		children[1] = new Chromosome(child2);
		if(GAMP.systemout) System.out.println("\nCrossover uniforme (1)");
		if(GAMP.systemout) System.out.println("Padre: " + daddy);
		if(GAMP.systemout) System.out.println("Madre: " + mummy);
		if(GAMP.systemout) System.out.println("Figlio1: " + children[0].getChromosome());
		if(GAMP.systemout) System.out.println("Figlio2: " + children[1].getChromosome());

		return children;
	}
	
	/**
	 * Crossover uniforme (2)
	 * Si calcola ogni gene di ogni discendente indipendentemente dagli altri 
	 * permettendo in tal modo ad un genitore di fornire lo stesso gene ad entrambi i figli
	 * @param daddy
	 * @param mummy
	 * @return children (array di due figli)
	 */
	public static Chromosome[] uniformCrossover2(Chromosome daddy, Chromosome mummy)
	{
		Chromosome[] children = new Chromosome[2];
		
		String child1 = "";
		String child2 = "";		
		for (int i = 0; i < daddy.getChromosome().length(); i++) 
		{
			double flipCoin = Math.random();
			if (flipCoin <= 0.5)
				child1 = child1 + daddy.getChromosome().charAt(i);
			else
				child1 = child1 + mummy.getChromosome().charAt(i);
			
			flipCoin = Math.random();
			if (flipCoin <= 0.5)
				child2 = child2 + daddy.getChromosome().charAt(i);
			else
				child2 = child2 + mummy.getChromosome().charAt(i);
		}
		
		children[0] = new Chromosome(child1);
		children[1] = new Chromosome(child2);
		if(GAMP.systemout) System.out.println("\nCrossover uniforme (2)");
		if(GAMP.systemout) System.out.println("Padre: " + daddy);
		if(GAMP.systemout) System.out.println("Madre: " + mummy);
		if(GAMP.systemout) System.out.println("Figlio1: " + children[0].getChromosome());
		if(GAMP.systemout) System.out.println("Figlio2: " + children[1].getChromosome());

		return children;
	}
}	