import java.io.IOException;
import java.util.*;

public class OptimumList
{
	/**
	 * Array di celle
	 */
	public class listaCelle	
	{
		SimulationData.MatrixCell[] mc;
		
		public listaCelle()
		{
			mc = new SimulationData.MatrixCell[100];
		}
	}
	
	/**
	 * Costruttore
	 */
	public OptimumList(){}
	
	/**
	 * Calcola percorso ottimo con propagazione della distanza a partire dal goal
	 * @return stringa di mosse
	 */
	public static String calcolaOttimo(String mappa)
	{
		//Carica matrice
		SimulationData newSimul;
		try {
			newSimul = new SimulationData(mappa);
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
		//Copia matrice	
		byte[][] matrix = new byte[newSimul.envMat.length][newSimul.envMat[0].length];
		for (int i=0; i < newSimul.envMat.length; i++)
			for (int j=0; j < newSimul.envMat[0].length; j++)
				matrix[i][j] = newSimul.envMat[i][j];
		//Array di array di celle
		listaCelle[] lista = new listaCelle[newSimul.envMat.length*newSimul.envMat[0].length];
		for (int i = 0; i < lista.length; i++)
			lista[i] = (new OptimumList()).new listaCelle();
						
		//Indici posizione iniziale		
		int riga = newSimul.getRobotStartPos().y;
		int colonna = newSimul.getRobotStartPos().x;
		
		//Indici posizione goal		
		int rigaGoal = newSimul.getRobotEndPos().y;
		int colonnaGoal = newSimul.getRobotEndPos().x;
		//in lista[i] metto tutte le celle che distano i dal goal 
		lista[0].mc[0] = newSimul.new MatrixCell(colonnaGoal, rigaGoal);
		//metto a 1 le celle che ho gi=E0 inserito nell'array
		matrix[rigaGoal][colonnaGoal] = 1;
		
		int i = 0;
		int j = 0;
		//Posizioni nell'array per la cella Start
		int iStart = 0;
		int jStart = 0;
		while (i < lista.length && lista[i].mc[0] != null)
		{
			int k = 0;
			while (j < lista[i].mc.length && lista[i].mc[j] != null)
			{
				//estraggo coordinate
				int c = lista[i].mc[j].x;
				int r = lista[i].mc[j].y;
				//vedo se sono arrivato allo start
				if (r == riga && c == colonna)
				{
					iStart = i;
					jStart = j;
				}
			
				//guardo cella a sinistra di (r,c) se esiste e se non  gi=E0 stata inserita e se non  un ostacolo
				//la inserisco nella posizione dell'array che significa distanza corrente + 1
				if (c-1 >= 0 && matrix[r][c-1] != 1)
				{
					lista[i+1].mc[k] = newSimul.new MatrixCell(c-1, r);
					matrix[r][c-1] = 1;
					k++;
				}
				//guardo cella a destra di (x,y)
				if (c+1 < newSimul.envMat[0].length && matrix[r][c+1] != 1)
				{
					lista[i+1].mc[k] = newSimul.new MatrixCell(c+1, r);
					matrix[r][c+1] = 1;
					k++;
				}
				//guardo cella sopra a (x,y)
				if (r-1 >= 0 && matrix[r-1][c] != 1)
				{
					lista[i+1].mc[k] = newSimul.new MatrixCell(c, r-1);
					matrix[r-1][c] = 1;
					k++;
				}
				//guardo cella sotto a (x,y)
				if (r+1 < newSimul.envMat.length && matrix[r+1][c] != 1)
				{
					lista[i+1].mc[k] = newSimul.new MatrixCell(c, r+1);
					matrix[r+1][c] = 1;
					k++;
				}
				j++;
			}
			i++;		
			j = 0;
		}
		//la cella start  salvata in lista[iStart).mc[jStart)
		
		/*Indica in quale direzione guarda il robot
		 * 0 = nord
		 * 1 = est
		 * 2 = sud
		 * 3 = ovest 
		 */
		int currDirection = newSimul.envMat[riga][colonna] - 2; // conversione da valore della cella di inizio a valore della direzione nel path
		// mappata nel range di valori [0, 4] che  quello dei valori accettati dal metodo drawArrow

		String bestChromosome = "";
		int newDirection = currDirection;
		boolean trovato = false;
		i = iStart;
		while (i > 0)
		{
			trovato = false;
			while (trovato == false && i < lista.length)
			{
				//Scorro l'array e vedo se esistono celle attigue alla posizione corrente che hanno distanza
				//minore rispetto a dove sono adesso
				if (currDirection == 0)
				{
					int h = 0;
					while (h < lista[i-1].mc.length && lista[i-1].mc[h] != null)
					{
						int xElem = lista[i-1].mc[h].x;
						int yElem = lista[i-1].mc[h].y;
						if (xElem == colonna+1 && yElem == riga)
						{
							bestChromosome += "D";
							newDirection = 1;
							trovato = true;
							colonna = xElem;
							break;
						}
						else if (xElem == colonna-1 && yElem == riga)
						{
							bestChromosome += "S";
							newDirection = 3;
							trovato = true;
							colonna = xElem;
							break;
						}
						else if (xElem == colonna && yElem == riga-1)
						{
							bestChromosome += "A";
							trovato = true;
							riga = yElem;
							break;
						}		
						h++;
					}
				}
				else if (currDirection == 1)
				{
					int h = 0;
					while (h < lista[i-1].mc.length && lista[i-1].mc[h] != null)
					{
						int xElem = lista[i-1].mc[h].x;
						int yElem = lista[i-1].mc[h].y;
						if (xElem == colonna && yElem == riga-1)
						{
							bestChromosome += "S";
							newDirection = 0;
							trovato = true;
							riga = yElem;
							break;
						}
						else if (xElem == colonna && yElem == riga+1)
						{
							bestChromosome += "D";
							newDirection = 2;
							trovato = true;
							riga = yElem;
							break;
						}
						else if (xElem == colonna+1 && yElem == riga)
						{
							bestChromosome += "A";
							trovato = true;
							colonna = xElem;
							break;
						}
						h++;
					}
				}
				else if (currDirection == 2)
				{
					int h = 0;
					while (h < lista[i-1].mc.length && lista[i-1].mc[h] != null)
					{
						int xElem = lista[i-1].mc[h].x;
						int yElem = lista[i-1].mc[h].y;
						if (xElem == colonna+1 && yElem == riga)
						{
							bestChromosome += "S";
							newDirection = 1;
							trovato = true;
							colonna = xElem;
							break;
						}
						else if	(xElem == colonna-1 && yElem == riga)
						{
							bestChromosome += "D";
							newDirection = 3;
							trovato = true;
							colonna = xElem;
							break;
						}
						else if	(xElem == colonna && yElem == riga+1)
						{
							bestChromosome += "A";
							riga = yElem;
							trovato = true;
							break;
						}					
						h++;
					}
				}
				if (currDirection == 3)
				{
					int h = 0;
					while (h < lista[i-1].mc.length && lista[i-1].mc[h] != null)
					{
						int xElem = lista[i-1].mc[h].x;
						int yElem = lista[i-1].mc[h].y;
						if (xElem == colonna && yElem == riga-1)
						{
							bestChromosome += "D";
							newDirection = 0;
							trovato = true;
							riga = yElem;
							break;
						}
						else if	(xElem == colonna && yElem == riga+1)
						{
							bestChromosome += "S";
							newDirection = 2;
							trovato = true;
							riga = yElem;
							break;
						}
						else if	(xElem == colonna-1 && yElem == riga)
						{
							bestChromosome += "A";
							trovato = true;
							colonna = xElem;
							break;
						}					
						h++;
					}
				}

				//Se non ho trovato nessuna cella con distanza minore a quella in cui sono adesso, significa
				//che la cella con distanza minore  quella dietro al robot, quindi devo permettergli di spostarsi
				//in una cella con distanza maggiore per girarsi
				if (trovato == false)
					i = i+2;
				currDirection = newDirection;
			}
			if (i >= lista.length)
			{
				bestChromosome = "";
				break;
			}
			i--;
		}
		
		if(GAMP.systemout)System.out.println("La soluzione ottima arriva al goal in " + bestChromosome.length() + " passi");
		if(GAMP.systemout)System.out.println(bestChromosome);
		
		return bestChromosome;
	}
}
