public class Reproduction {
	/**
	 * Simula il processo riproduttivo-genetico complessivo
	 * 
	 * @param generation
	 * @param nGenerations
	 * @param selectionType
	 * @param crossoverType
	 * @param crossoverProb
	 * @param mutationType
	 * @param mutationProb
	 * @return
	 */
	public static Chromosome[] geneticProcess(Chromosome[] generation, int nGeneration, int selectionType, int crossoverType, double crossoverProb,
			int mutationType, double mutationProb) {

		int nIndiv; // numero di individui
		if (generation != null)
			nIndiv = generation.length;
		else
			nIndiv = 0;
		if (nIndiv == 0)
			return null;

		Chromosome[] matingPool = generation;

		// selezione-riproduzione
		if (GAMP.systemout)
			if (GAMP.systemout)
				System.out.println("\nCreazione mating pool");
		matingPool = selection(matingPool, selectionType);
		if (GAMP.systemout)
			System.out.println();
		if (GAMP.systemout)
			System.out.println("Mating pool generato:");
		for (int j = 0; j < matingPool.length; j++)
			if (GAMP.systemout)
				System.out.println(matingPool[j].getChromosome());
		if (GAMP.systemout)
			System.out.println();

		// crossover
		if (GAMP.systemout)
			System.out.println("Fase di crossover");
		generation = crossover(matingPool, crossoverType, crossoverProb);
		if (GAMP.systemout)
			System.out.println();
		if (GAMP.systemout)
			System.out.println("Popolazione dopo crossover");
		for (int j = 0; j < generation.length; j++)
			if (GAMP.systemout)
				System.out.println(generation[j].getChromosome());

		// mutazioni
		if (GAMP.systemout)
			System.out.println("\nMutazioni eseguite con probabilit� " + mutationProb + "\n");
		generation = mutation(generation, mutationType, mutationProb);
		if (GAMP.systemout)
			System.out.println();
		if (GAMP.systemout)
			System.out.println("Generazione " + nGeneration + " dopo crossover e mutazioni");
		for (int j = 0; j < generation.length; j++)
			if (GAMP.systemout)
				System.out.println(generation[j].getChromosome());
		if (GAMP.systemout)
			System.out.println();

		return generation;
	}

	/**
	 * E' il metodo che seleziona quali cromosomi della generazione corrente
	 * inserire nel mating pool di riproduzione Lo fa in modo da inserire con
	 * maggiore probabilit nel pool riproduttivo gli individui che hanno
	 * fitness function migliore
	 * 
	 * @param matingPool
	 *            generazione di input
	 * @param selectionType
	 *            tipo di selezione (1: proporzionale, 2: tournment, 3: rank
	 *            based)
	 * @return matingPool finale su cui poi andare ad operare con crossover
	 */
	private static Chromosome[] selection(Chromosome[] matingPool, int selectionType) {
		Chromosome[] finalMatingPool = new Chromosome[matingPool.length];

		// selezione proporzionale
		if (selectionType == 1) {
			double sumFitness = 0;
			double fakeFitness = 0; // per implementare la roulette
			// calcolo della somma della fitness
			for (int i = 0; i < matingPool.length; i++)
				sumFitness += matingPool[i].getFitness();
			if (GAMP.systemout)
				System.out.println("Somma fitness: " + sumFitness);

			// poolTemp ha come fitness la probabilit di selezione cumulativa
			Chromosome[] poolTemp = matingPool;
			if (GAMP.systemout)
				System.out.println();
			if (GAMP.systemout)
				System.out.println("Mating pool temporaneo (fake!)");
			for (int i = 0; i < matingPool.length; i++) {
				fakeFitness += (matingPool[i].getFitness() / sumFitness);
				poolTemp[i].setFitness(fakeFitness);
				if (GAMP.systemout)
					System.out.println(i + ")" + poolTemp[i]);
			}

			// creo il matingPool finale
			finalMatingPool = new Chromosome[matingPool.length];
			for (int i = 0; i < matingPool.length; i++) {
				// simulazione della roulette
				double rand = Math.random();
				if (GAMP.systemout)
					System.out.println("Roulette random: " + rand);
				fakeFitness = 0;
				int j = 0;
				do {
					if (j == poolTemp.length)
						break;
					fakeFitness = poolTemp[j].getFitness();
					if (GAMP.systemout)
						System.out.println("Fakefitness:" + fakeFitness);
					j++;
				} while (fakeFitness < rand);
				if (GAMP.systemout)
					System.out.println("Mi sono fermato a j=" + j);

				// seleziona il gene individuato dalla roulette
				// al mating pool finale
				if (GAMP.systemout)
					System.out.println("Scelgo cromosoma: (" + (j - 1) + ") " + poolTemp[j - 1].getChromosome());
				finalMatingPool[i] = poolTemp[j - 1];
			}
			return finalMatingPool;
		}

		// selezione a gironi (tournament selection)
		if (selectionType == 2) {
			int tournamentSize = 1; // ampiezza di girone
			int tournamentNumber = matingPool.length; // numero di gironi da
														// eseguire
			int matingPoolLength = matingPool.length;

			// calcolo del numero di individui per girone
			if (matingPoolLength >= 40)
				tournamentSize = 5;
			else if (matingPoolLength >= 20)
				tournamentSize = 4;
			else if (matingPoolLength >= 10)
				tournamentSize = 3;
			else
				tournamentSize = 2; //  una condizione che non dovrebbe essere
									// necessaria
			if (GAMP.systemout)
				System.out.println("*** Individui per girone:" + tournamentSize);
			// esegue tournamentNumber gironi
			for (int i = 0; i < tournamentNumber; i++) {
				if (GAMP.systemout)
					System.out.println("***** Generazione tournament:" + i);
				// crea una copia del matingPool
				Chromosome[] tempMatingPool = new Chromosome[tournamentNumber];
				for (int h = 0; h < tempMatingPool.length; h++)
					tempMatingPool[h] = matingPool[h];

				// istanzia un tournament vuoto
				Chromosome[] tournament = new Chromosome[tournamentSize]; 

				int j = 0;
				while (j < tournamentSize) {
					int rand = (int) (Math.random() * matingPool.length);
					if (GAMP.systemout)
						System.out.println("** Rand:" + rand + " j:" + j + " tournamentSize" + tournamentSize);
					if (tempMatingPool[rand] != null) {
						tournament[j] = tempMatingPool[rand];
						tempMatingPool[rand] = null;
						j++;
					}
				}
				// girone completato
				Chromosome theBest = tournament[0]; // inizializzo il migliore
													// al primo che trovo
				// cerco il migliore nel girone partendo dal secondo
				for (int k = 1; k < tournamentSize; k++) {
					if (tournament[k].getFitness() >= theBest.getFitness())
						theBest = tournament[k]; // se migliore, aggiorno best
				}
				// fine dell'uso del girone... salvo il migliore come nuovo
				// elemento del matingPoolFinale
				if (GAMP.systemout)
					System.out.println("*** BestOne:" + theBest);
				finalMatingPool[i] = theBest;
			}
			return finalMatingPool;
		}

		// selezione rank-based
		if (selectionType == 3) {
			double probSum = 0;

			// crea una copia del matingPool
			Chromosome[] tempMatingPool = new Chromosome[matingPool.length];
			for (int h = 0; h < tempMatingPool.length; h++)
				tempMatingPool[h] = matingPool[h];
			// d'ora in poi lavoro solo sul tempMatingPool

			// sorting con MergeSort iterativo
			iterativeMergeSortOnChromosomes(tempMatingPool);

			if (GAMP.systemout)
				System.out.println("Mating pool ordinato in base alla fitness:");
			for (int h = 0; h < tempMatingPool.length; h++)
				if (GAMP.systemout)
					System.out.println(tempMatingPool[h]);
			// calcolo della probabilit per ciascun cromosoma con x^2+1
			for (int h = 0; h < tempMatingPool.length; h++) {
				double fakeFitness = Math.pow(h, 2) + 1;
				if (GAMP.systemout)
					System.out.println(fakeFitness);
				tempMatingPool[h].setFitness(fakeFitness);
				if (GAMP.systemout)
					System.out.println(tempMatingPool[h]);
			}

			// calcolo della probabilit totale
			for (int h = 0; h < tempMatingPool.length; h++)
				probSum += tempMatingPool[h].getFitness();
			// normalizzazione delle probabilit in modo che sommi a 1
			for (int h = 0; h < tempMatingPool.length; h++)
				tempMatingPool[h].setFitness(tempMatingPool[h].getFitness() / probSum);

			// controllo che sommi a 1
			probSum = 0;
			if (GAMP.systemout)
				System.out.println("Probabilit ordinate:");
			for (int h = 0; h < tempMatingPool.length; h++) {
				if (GAMP.systemout)
					System.out.println(tempMatingPool[h]);
				probSum += tempMatingPool[h].getFitness();
			}
			if (GAMP.systemout)
				System.out.println("Somma probabilit: " + probSum);
			// fake fitness (probabilit) cumulativa per individuare gli
			// intervalli per la roulette
			double fakeFitness = 0;
			for (int i = 0; i < tempMatingPool.length; i++) {
				fakeFitness += tempMatingPool[i].getFitness();
				tempMatingPool[i].setFitness(fakeFitness);
			}
			if (GAMP.systemout)
				System.out.println("Intervalli di probabilit:");
			for (int h = 0; h < tempMatingPool.length; h++)
				if (GAMP.systemout)
					System.out.println(tempMatingPool[h]);

			// roulette vera e propria
			// creo il matingPool finale
			finalMatingPool = new Chromosome[matingPool.length];
			for (int i = 0; i < tempMatingPool.length; i++) {
				// simulazione della roulette
				double rand = Math.random();
				if (GAMP.systemout)
					System.out.println("Roulette random: " + rand);
				fakeFitness = 0;
				int j = 0;
				do {
					if (j == tempMatingPool.length)
						break;
					fakeFitness = tempMatingPool[j].getFitness();
					if (GAMP.systemout)
						System.out.println("Fakefitness:" + fakeFitness);
					j++;
				} while (fakeFitness < rand);
				if (GAMP.systemout)
					System.out.println("Mi sono fermato a j=" + j);

				// seleziona il gene individuato dalla roulette al mating pool
				// finale
				if (GAMP.systemout)
					System.out.println("Scelgo cromosoma: (" + (j - 1) + ") " + tempMatingPool[j - 1].getChromosome());
				finalMatingPool[i] = tempMatingPool[j - 1];
			}
			return finalMatingPool;
		} else
			return finalMatingPool;
	}

	private static Chromosome[] crossover(Chromosome[] matingPool, int crossoverType, double crossoverProb) {
		Chromosome[] generation = new Chromosome[matingPool.length];
		Chromosome[] children = new Chromosome[2]; // inizializzato

		for (int i = 0; i < matingPool.length; i += 2) {
			// seleziono casualmente i genitori
			int rand = (int) (Math.random() * matingPool.length);
			while (matingPool[rand] == null)
				rand = (int) (Math.random() * matingPool.length);
			Chromosome daddy = matingPool[rand];
			matingPool[rand] = null;
			int index = rand;
			while (matingPool[rand] == null)
				rand = (int) (Math.random() * matingPool.length);
			Chromosome mummy = matingPool[rand];
			matingPool[rand] = null;

			double randomValue = Math.random();
			if (randomValue > crossoverProb) {
				children[0] = daddy;
				children[1] = mummy;
				if (GAMP.systemout)
					System.out.println("Nessun crossover tra cromosomi " + index + " e " + rand);
			} else {
				if (GAMP.systemout)
					System.out.println("Crossover tra cromosomi " + index + " e " + rand);
				if (crossoverType == 1)
					children = Chromosome.crossoverOnePoint(daddy, mummy);
				if (crossoverType == 2)
					children = Chromosome.crossoverTwoPoints(daddy, mummy);
				if (crossoverType == 3)
					children = Chromosome.uniformCrossover1(daddy, mummy);
				if (crossoverType == 4)
					children = Chromosome.uniformCrossover2(daddy, mummy);
			}
			generation[i] = children[0];
			generation[i + 1] = children[1];
		}
		return generation;
	}

	private static Chromosome[] mutation(Chromosome[] generation, int mutationType, double mutationProb) {
		for (int j = 0; j < generation.length; j++) {
			if (GAMP.systemout)
				System.out.println("Mutazione sull'individo " + j);
			if (mutationType == 1)
				generation[j].mutation_simple(mutationProb);
			if (mutationType == 2)
				generation[j].mutation_simple2(mutationProb);
			if (mutationType == 3)
				generation[j].mutation_inverse(mutationProb);
			if (GAMP.systemout)
				System.out.println();
		}
		return generation;
	}

	/**
	 * Il metodo implementa l'algorimto di MergeSort iterativo su oggetti
	 * Chromosome
	 * 
	 * @param orig
	 *            array di Chromosome che contiene la sequenza da ordinare
	 */
	private static void iterativeMergeSortOnChromosomes(Chromosome[] orig) {

		Chromosome[] in = new Chromosome[orig.length];
		System.arraycopy(orig, 0, in, 0, in.length);
		Chromosome[] out = new Chromosome[in.length];
		Chromosome[] temp;
		int n = in.length;
		for (int i = 1; i < n; i *= 2) {
			for (int j = 0; j < n; j += 2 * i)
				iterativeMerge(in, out, j, i);
			temp = in;
			in = out;
			out = temp;
		}
		System.arraycopy(in, 0, orig, 0, in.length);
	}// iterativeMergeSortOnChromosomes

	/**
	 * Il metodo realizza la fusione (merge) dell'algoritmo iterativo
	 * 
	 * @param in
	 *            array di partenza
	 * @param out
	 *            array di arrivo
	 * @param start
	 *            posizione di inizio fusione
	 * @param inc
	 *            incremento
	 */
	private static void iterativeMerge(Chromosome[] in, Chromosome[] out, int start, int inc) {
		int x = start;
		int end1 = Math.min(start + inc, in.length);
		int end2 = Math.min(start + 2 * inc, in.length);
		int y = start + inc;
		int z = start;
		while ((x < end1) && (y < end2))
			if (in[x].getFitness() <= in[y].getFitness()) {
				Chromosome ch = new Chromosome(in[x++].getChromosome());
				ch.setFitness(in[x - 1].getFitness());
				out[z++] = ch;

			} else {// out[z++]=in[y++];
				Chromosome ch = new Chromosome(in[y++].getChromosome());
				ch.setFitness(in[y - 1].getFitness());
				out[z++] = ch;
			}

		if (x < end1)
			System.arraycopy(in, x, out, z, end1 - x);
		else if (y < end2)
			System.arraycopy(in, y, out, z, end2 - y);
	}// iterativeMerge
}
