

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;

import javax.comm.CommPortIdentifier;
import javax.comm.SerialPort;

import communication.transport.ConnectionProvider;

/**
 * @author Paolo
 * 
 */
public class SerialJavaxCommProvider implements ConnectionProvider
{
	private SerialPort port;

	private String serialDevice;

	private int baudRate;

	private InputStream in;

	private OutputStream out;

	/**
	 * Create a serial connection provider
	 * 
	 * @param serialDevice
	 *            Device name
	 * @param baudRate
	 *            Communication speed
	 */
	public SerialJavaxCommProvider(String serialDevice, int baudRate)
	{
		this.serialDevice = serialDevice;
		this.baudRate = baudRate;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see communication.transport.ConnectionProvider#getInputStream()
	 */
	public InputStream getInputStream()
	{
		return in;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see communication.transport.ConnectionProvider#getOutputStream()
	 */
	public OutputStream getOutputStream()
	{
		return out;
	}

	public int getStreamProperties()
	{
		// TODO controllare...
		return 0;// ConnectionProvider.AVAILABLE_IMPLEMENTED;
	}

	public boolean openConnection() throws IOException
	{
		CommPortIdentifier portId;
		try
		{
			portId = CommPortIdentifier.getPortIdentifier(serialDevice);
			if (portId == null)
				return false;
			port = (SerialPort) portId.open(getClass().getName(), 5000);
			if (port == null)
				return false;
			port.setSerialPortParams(baudRate, SerialPort.DATABITS_8,
				SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
			port.setFlowControlMode(SerialPort.FLOWCONTROL_RTSCTS_IN
				| SerialPort.FLOWCONTROL_RTSCTS_OUT);
		}
		catch (Exception e)
		{
			throw new IOException(e.getMessage());
		}
		in = port.getInputStream();
		if (in == null)
			return false;
		out = port.getOutputStream();
		if (out == null)
			return false;
		return true;
	}

	public void closeConnection() throws IOException
	{
		IOException ex = null;
		if (out != null)
		{
			try
			{
				out.close();
				out = null;
			}
			catch (IOException e)
			{
				ex = e;
			}
		}
		if (in != null)
		{
			try
			{
				in.close();
				in = null;
			}
			catch (IOException e)
			{
				ex = e;
			}
		}
		if (port != null)
		{
			port.close();
			port = null;
		}
		if (ex != null)
			throw ex;
	}

	public static Iterator getPortList()
	{
		ArrayList list = new ArrayList();
		CommPortIdentifier portId;
		Enumeration portList = CommPortIdentifier.getPortIdentifiers();
		while (portList.hasMoreElements())
		{
			portId = (CommPortIdentifier) portList.nextElement();
			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL)
				list.add(portId.getName());
		}
		return list.iterator();
	}
}
