import java.io.*;
import java.util.ArrayList;

public class SimulationData {

	/**
	 * Rappresenta l'indice di una cella della matrice dell'ambiente di
	 * movimento del robot
	 * 
	 * @author marco
	 * 
	 */
	public class MatrixCell {
		public int x;
		public int y;

		public MatrixCell() {
			this.x = 1;
			this.y = 1;
		}

		public MatrixCell(int x, int y) {
			this.x = x;
			this.y = y;
		}

		public String toString() {
			return "x: " + x + "\ty: " + y;
		}
	}

	public int currGeneration = 0; // numero generazione corrente
	private int nGenes = 0; // numero di geni per individuo
	public static int nIndiv = 0; // numero di individui
	private int nGenerations = 0; // numero di generazioni

	public int flag = 0; // flag per interrompere il ciclo in caso di soluzione
							// trovata

	// Una generazione e' vista come un array di nIndiv oggetti di tipo
	// Chromosome
	public Chromosome[] runningGeneration;
	private Chromosome solution;
	public ArrayList allGenerations;
	private MatrixCell robotStartPos;// =new MatrixCell();
	private MatrixCell robotEndPos;// =new MatrixCell();

	/**
	 * matrice che rappresenta l'ambiente di movimento del robot. I suoi valori
	 * sono byte con la seguente convenzione per i valori che possono assumere:
	 * 0 - cella vuota 1 - cella con ostacolo 2 - cella della posizione iniziale
	 * con robot orientato a N 3 - cella della posizione iniziale con robot
	 * orientato a S 4 - cella della posizione iniziale con robot orientato a W
	 * 5 - cella della posizione iniziale con robot orientato a E 6 - cella del
	 * goal da raggiungere 7 - cella del percorso con freccia orientata a N 8 -
	 * cella del percorso con freccia orientata a S 9 - cella del percorso con
	 * freccia orientata a W 10 - cella del percorso con freccia orientata a E
	 */
	public byte[][] envMat;

	// Valori per la matrice envMat
	public class matValues {
		public static final byte EMPTY = 0;
		public static final byte OBSTACLE = 1;
		public static final byte BEGIN_N = 2;
		public static final byte BEGIN_E = 3;
		public static final byte BEGIN_S = 4;
		public static final byte BEGIN_W = 5;
		public static final byte GOAL = 6;
		public static final byte PATH_N = 7;
		public static final byte PATH_E = 8;
		public static final byte PATH_S = 9;
		public static final byte PATH_W = 10;
		public static final byte DEATH = -1;
	}
	
	public void delete(){
		envMat=null;
		allGenerations=null;
	}

	public int getNGenes() {
		return nGenes;
	}

	public void setNGenes(int genes) {
		nGenes = genes;
	}

	public int getNGenerations() {
		return nGenerations;
	}

	public void setNGenerations(int generations) {
		nGenerations = generations;
	}

	public Chromosome getSolution() {
		if (flag == 1)
			return solution;

		return null;
	}

	public void setSolution(Chromosome solution) {
		this.solution = solution;
	}

	public int getNIndiv() {
		return nIndiv;
	}

	public void setNIndiv(int indiv) {
		nIndiv = indiv;
	}

	public double getDistance(int x, int y, MatrixCell b) {
		if (x >= 0 && x < envMat[0].length && y >= 0 && y < envMat.length)
			return (double) getDistance(new MatrixCell(x, y), b);
		else if (x == -1)
			return (double) getDistance(new MatrixCell(0, y), b);
		if (x == envMat[0].length)
			return (double) getDistance(new MatrixCell(envMat[0].length - 1, y), b);
		if (y == -1)
			return (double) getDistance(new MatrixCell(x, 0), b);
		if (y == envMat.length)
			return (double) getDistance(new MatrixCell(x, envMat.length - 1), b);
		return (double) distanza(y, x, b.y, b.x);
	}

	public double getDistance(int x, int y, int z, int w) {
		if (x >= 0 && x < envMat[0].length && y >= 0 && y < envMat.length && z >= 0 && z < envMat[0].length & w >= 0 && w < envMat.length)
			return (double) getDistance(new MatrixCell(x, y), new MatrixCell(z, w));
		else {
			if (x == -1)
				return (double) getDistance(new MatrixCell(0, y), new MatrixCell(z, w));
			if (x == envMat[0].length)
				return (double) getDistance(new MatrixCell(envMat[0].length - 1, y), new MatrixCell(z, w));
			if (y == -1)
				return (double) getDistance(new MatrixCell(x, 0), new MatrixCell(z, w));
			if (y == envMat.length)
				return (double) getDistance(new MatrixCell(x, envMat.length - 1), new MatrixCell(z, w));
		}
		return (double) distanza(y, x, z, w);
	}

	/**
	 * Calcola la distanza col teorema di Pitagora
	 * 
	 * @param rAttuale
	 * @param cAttuale
	 * @param rGoal
	 * @param cGoal
	 * @return
	 */
	public static double distanza(int rAttuale, int cAttuale, int rGoal, int cGoal) {
		return Math.sqrt(Math.pow((rAttuale - rGoal), 2) + Math.pow((cAttuale - cGoal), 2));
	}

	public int getDistance(MatrixCell a, MatrixCell b) {
		int c, d;
		int e = 0;
		int direction = getMatrixValue(a);

		// Se la posizione iniziale e' verso Sud e la posizione finale e' a Nord
		// di quella iniziale il robot dovrà percorrere quattro caselle in più
		if (a.x == b.x && a.y > b.y && (direction == matValues.BEGIN_S || direction == matValues.PATH_S)) {
			e += 2;
		}
		// Se la posizione iniziale e' verso Nord e la posizione finale e' a Sud
		// di quella iniziale il robot dovrà percorrere quattro caselle in più
		if (a.x == b.x && a.y < b.y && (direction == matValues.BEGIN_N || direction == matValues.PATH_N)) {
			e += 2;
		}
		// Se la posizione iniziale è verso Est e la posizione finale è a
		// Ovest quella iniziale il robot dovrà percorrere quattro caselle in
		// più
		if (a.y == b.y && a.x > b.x && (direction == matValues.BEGIN_E || direction == matValues.PATH_E)) {
			e += 2;
		}
		// Se la posizione iniziale è verso Est e la posizione finale è a
		// Ovest quella iniziale il robot dovrà percorrere quattro caselle in
		// più
		if (a.y == b.y && a.x < b.x && (direction == matValues.BEGIN_W || direction == matValues.PATH_W)) {
			e += 2;
		}

		if (a.x > b.x) {
			c = a.x - b.x;
		} else {
			c = b.x - a.x;
		}
		if (a.y > b.y) {
			d = a.y - b.y;
		} else {
			d = b.y - a.y;
		}

		return c + d + e;
	}

	// public boolean isStartCollide(int x, int y) {
	// return isStartCollide(new MatrixCell(x, y));
	// }
	//
	// public boolean isStartCollide(MatrixCell cell) {
	// // Controllo che non sia sul bordo
	// if ((cell.y == 0 && getMatrixValue(cell) == matValues.BEGIN_N) || (cell.x
	// == 0 && getMatrixValue(cell) == matValues.BEGIN_W)
	// || (cell.y == envMat.length - 1 && getMatrixValue(cell) ==
	// matValues.BEGIN_S)
	// || (cell.x == envMat[0].length - 1 && getMatrixValue(cell) ==
	// matValues.BEGIN_E))
	// return true;
	// return false;
	// }

	// public boolean isObstacleCollide(int x, int y) {
	// return isObstacleCollide(new MatrixCell(x, y));
	// }
	//
	// public boolean isObstacleCollide(MatrixCell cell) {
	// // controllo che non ci siano ostacoli
	// if (getMatrixValue(cell) == matValues.OBSTACLE && envMat[0].length >=
	// cell.x + 2)
	// if (getMatrixValue(cell.x + 1, cell.y) == matValues.BEGIN_W)
	// return true;
	// if (getMatrixValue(cell) == matValues.OBSTACLE && envMat.length >= cell.y
	// + 2)
	// if (getMatrixValue(cell.x, cell.y + 1) == matValues.BEGIN_N)
	// return true;
	// if (getMatrixValue(cell) == matValues.OBSTACLE && cell.x - 1 >= 0)
	// if (getMatrixValue(cell.x - 1, cell.y) == matValues.BEGIN_E)
	// return true;
	// if (cell.y - 1 >= 0 && getMatrixValue(cell) == matValues.OBSTACLE)
	// if (getMatrixValue(cell.x, cell.y - 1) == matValues.BEGIN_S)
	// return true;
	// return false;
	// }

	public int getMatrixValue(MatrixCell cell) {
		return envMat[cell.y][cell.x];
	}

	public int getMatrixValue(int x, int y) {
		return envMat[y][x];
	}

	public void setMatrixValue(MatrixCell cell, int i) {
		envMat[cell.y][cell.x] = (byte) i;
	}

	public void setMatrixValue(int x, int y, int i) {
		envMat[y][x] = (byte) i;
	}

	/**
	 * Restituisce gli indici nella matrice dell'ambiente della posizione di
	 * start come oggetto di tipo MatrixCell
	 * 
	 * @return
	 */
	public MatrixCell getRobotStartPos() {
		return robotStartPos;
	}

	public void setRobotStartPos(MatrixCell cell) {
		setRobotStartPos(cell.x, cell.y);
	}

	public void setRobotStartPos(int x, int y) {
		if (x >= 0 && y >= 0 && x < envMat[0].length && y < envMat.length) {
			robotStartPos = new MatrixCell();
			robotStartPos.x = x;
			robotStartPos.y = y;
		}
	}

	public void setRobotEndPos(MatrixCell cell) {
		setRobotEndPos(cell.x, cell.y);
	}

	public void setRobotEndPos(int x, int y) {
		if (x >= 0 && y >= 0 && x < envMat[0].length && y < envMat.length) {
			robotEndPos = new MatrixCell();
			robotEndPos.x = x;
			robotEndPos.y = y;
		}
	}

	/**
	 * Restituisce gli indici nella matrice dell'ambiente della posizione del
	 * goal come oggetto di tipo MatrixCell
	 * 
	 * @return
	 */
	public MatrixCell getRobotEndPos() {
		return robotEndPos;
	}

	public SimulationData() {
	}

	public SimulationData(String fileName) throws IOException {
		loadMatrix(fileName);
	}

	/**
	 * Inizializza la matrice che costituisce l'ambiente di movimento del robot
	 * 
	 * @param topLeft
	 * @param bottomDown
	 * @return
	 */
	public void initMatrix(int width, int height) {
		envMat = new byte[height][width];
		for (int i = 0; i < width; i++)
			for (int j = 0; j < height; j++)
				if (i == 0 || j == 0 || j == height - 1 || i == width - 1)
					envMat[j][i] = matValues.OBSTACLE;
	}

	/**
	 * Salva la matrice dell'ambiente nel file specificato come argomento
	 * 
	 * @param fileName
	 */
	public void saveMatrix(String fileName) {
		try {
			PrintStream out = new PrintStream(fileName);
			String line = "";
			out.println(envMat.length);
			out.println(envMat[0].length);

			if (GAMP.systemout)
				System.out.println("mat.length: " + envMat.length + " mat[1].length: " + envMat[0].length);
			for (int i = 0; i < envMat.length; i++) {
				line = "";
				for (int j = 0; j < envMat[i].length; j++) {
					line += envMat[i][j] + " ";
				}
				out.println(line.trim());
			}
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Carica la matrice dell'ambiente dal file specificato come argomento e
	 * inizializza le variabili locali dell'oggetto sul quale è stata chiamata
	 * la funzione.
	 * 
	 * @param fileName
	 * @return
	 * @throws IOException
	 */
	public byte[][] loadMatrix(String fileName) throws IOException {
		byte[][] mat = null;
		try {
			BufferedReader in = new BufferedReader(new FileReader(fileName));
			String line = in.readLine();
			String line2 = in.readLine();
			mat = new byte[Integer.parseInt(line)][Integer.parseInt(line2)];
			envMat = mat; // spostata qui
			line = in.readLine();
			int i = 0;
			while (line != null) {
				String[] temp = line.split(" ");
				for (int j = 0; j < temp.length; j++) {
					mat[i][j] = Byte.parseByte(temp[j]);
					if (mat[i][j] >= matValues.BEGIN_N && mat[i][j] <= matValues.BEGIN_W) {
						setRobotStartPos(j, i);
					} else if (mat[i][j] == matValues.GOAL) {
						setRobotEndPos(j, i);
					}
				}
				i++;
				line = in.readLine();
			}
			in.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return mat;
	}

	public void initSimulation() {
		if (nGenes <= 0 || nIndiv <= 0) {
			if (GAMP.systemout)
				System.out.println("Variabili non completamente inizializzate.");
			return;
		}
		allGenerations = new ArrayList();
		runningGeneration = new Chromosome[nIndiv];
		flag = 0;
		// Creo popolazione iniziale
		if (GAMP.systemout)
			System.out.println("Popolazione iniziale:");
		for (int i = 0; i < runningGeneration.length; i++) {
			runningGeneration[i] = new Chromosome(nGenes);
			if (GAMP.systemout)
				System.out.println(runningGeneration[i].getChromosome());
		}
		if (GAMP.systemout)
			System.out.println();

		// Valutazione della generazione 0 (popolazione iniziale) nell'ambiente
		for (int i = 0; i < runningGeneration.length; i++) {
			try {
				double fit1=Move.steps(runningGeneration[i], i, 0,GAMP.test );
				runningGeneration[i].setFitnessMin(fit1);
				double fit=runningGeneration[i].getFitnessMin();
				if(GAMP.systemout)System.out.println(">>>>>>>"+fit1+">>"+fit+"->"+(1/fit));
				if(fit==10000)
				runningGeneration[i].setFitness(fit);
				else
					runningGeneration[i].setFitness(1/fit);
				
			} catch (IOException e) {
				e.printStackTrace();
			}
			// imposto un flag nel caso ci sia gia' una soluzione nella
			// popolazione iniziale
			if (runningGeneration[i].getFitness() == 10000) {
				flag = 1;
				solution = runningGeneration[i];
			}

		}
		// Stampa dei cromosomi
		if (GAMP.systemout)
			System.out.println();
		if (GAMP.systemout)
			System.out.println("Valore di fitness");
		double fmax = 0;
		for (int i = 0; i < runningGeneration.length; i++) {
			if (GAMP.systemout)
				System.out.println(runningGeneration[i]);
			if (runningGeneration[i].getFitness() > fmax)
				fmax = runningGeneration[i].getFitness();
			// if(generation[i].getFitness()>1000) break;
		}
		currGeneration = 1;
		nGenerations = 1;

		Chromosome[] newGeneration = new Chromosome[nIndiv];
		System.arraycopy(runningGeneration, 0, newGeneration, 0, runningGeneration.length);
		allGenerations.add(newGeneration);
	}

	public void calcNextGenerations(int step, int selectionType, int crossoverType, double crossoverProb, int mutationType, double mutationProb) {
		Chromosome[] newGeneration = null;
		// CICLO PRINCIPALE PER LE GENERAZIONI SUCCESSIVE ALLA PRIMA
		if (flag == 0) {// se non ho ancora una soluzione, allora genero le
			// generazioni successive
			for (currGeneration = nGenerations; currGeneration < nGenerations + step; currGeneration++) {
				if(GAMP.stop){break;}
				if (flag == 1 || flag == 2)
					break;// condizione di uscita dal ciclo nel caso abbia gi�
				// trovato soluzioni
				if (GAMP.systemout)
					System.out.println("\nSoluzione non trovata alla generazione " + (currGeneration - 1));
				if (GAMP.systemout)
					System.out.println();
				if (GAMP.systemout)
					System.out.println("=========================================================================");
				if (GAMP.systemout)
					System.out.println("Generazione " + currGeneration + " -------------------------------------------------------");
				if (GAMP.systemout)
					System.out.println("=========================================================================");

				runningGeneration = Reproduction.geneticProcess(runningGeneration, currGeneration, selectionType, crossoverType, crossoverProb, mutationType,
						mutationProb);

				// Valutazione della generazione k nell'ambiente
				for (int i = 0; i < runningGeneration.length; i++) {
					try {
						double fit1=Move.steps(runningGeneration[i], i,currGeneration, GAMP.test);
						runningGeneration[i].setFitnessMin(fit1);
						double fit=runningGeneration[i].getFitnessMin();
						if(GAMP.systemout)System.out.println(">>>>>>>"+fit1+">>"+fit+"->"+(1/fit));
						if(fit==10000)
						runningGeneration[i].setFitness(fit);
						else
							runningGeneration[i].setFitness(1/fit);
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
				if (GAMP.systemout)
					System.out.println("Valore di fitness");
				double fmax = 0;
				for (int i = 0; i < runningGeneration.length; i++) {
					if (runningGeneration[i].getFitness() > fmax)
						fmax = runningGeneration[i].getFitness();
					if (GAMP.systemout)
						System.out.println(runningGeneration[i]);
					if (runningGeneration[i].getFitness() == 10000) {
						flag = 1;
						solution = runningGeneration[i];
					}
				}

				newGeneration = new Chromosome[nIndiv];
				System.arraycopy(runningGeneration, 0, newGeneration, 0, runningGeneration.length);
				allGenerations.add(newGeneration);
			}
		}

		nGenerations += step;

		if (GAMP.systemout)
			System.out.println();
		if (GAMP.systemout)
			System.out.println();
		for (int i = 0; i < runningGeneration.length; i++) {
			if (runningGeneration[i].getFitness() == 10000)
				if (GAMP.systemout)
					System.out.println("Soluzione:" + runningGeneration[i] + " ottenuta alla generazione " + (currGeneration - 1));
		}
		if (currGeneration == nGenerations && flag != 1)
			if (GAMP.systemout)
				System.out.println("Soluzione non trovata dopo " + nGenerations + " generazioni");
	}
}
