/**
 * 
 */
package communication.handler;

import communication.protocol.ChunkReader;
import communication.protocol.ChunkWriter;

/**
 * Basic implementation for capability handlers
 * 
 * @author Paolo
 */
public abstract class AbstractCapabilityHandler implements CapabilityHandler
{
	/**
	 * The chunk writer
	 */
	protected ChunkWriter writer = null;

	/**
	 * The chunk reader
	 */
	protected ChunkReader reader = null;

	/**
	 * A flag to indicate cancellation.
	 * 
	 * @see communication.handler.CapabilityHandler#cancelHandler
	 */
	protected volatile boolean cancelled = true;

	/**
	 * Sub-classes must call the super-class implementation, perform additional
	 * initialization tasks, such as registering the capability listener, and
	 * return the appropriate capability identifier.
	 * 
	 * @see communication.handler.CapabilityHandler#initializeHandler
	 */
	public int initializeHandler(ChunkReader reader, ChunkWriter writer)
	{
		this.reader = reader;
		this.writer = writer;
		cancelled = false;
		return 0;
	}

	/**
	 * Sub-classes must call the super-class implementation and abort any
	 * pending operation.
	 * 
	 * @see communication.handler.CapabilityHandler#cancelHandler
	 */
	public void cancelHandler()
	{
		cancelled = true;
	}

	/**
	 * Sub-classes must perform additional clean-up tasks and call the
	 * super-class implementation for safety.
	 * 
	 * @see communication.handler.CapabilityHandler#disposeHandler
	 */
	public void disposeHandler()
	{
		reader = null;
		writer = null;
		cancelled = true;
	}

}
