/**
 * 
 */
package communication.handler;

import communication.protocol.Capability;
import communication.protocol.ChunkReader;
import communication.protocol.ChunkWriter;

/**
 * Interface to implement handlers for protocol capabilities
 * 
 * @author Paolo
 */
public interface CapabilityHandler
{
	/**
	 * Initialization routine. The handler should register its protocol listener
	 * interface to the reader and save a reference to both the chunk reader and
	 * writer for later use.
	 * 
	 * @param reader
	 *            The chunk reader for listener registration
	 * @param writer
	 *            The chunk writer for protocol interaction
	 * @return An integer that specifies the capability implemented by this
	 *         handler
	 * @see Capability
	 */
	int initializeHandler(ChunkReader reader, ChunkWriter writer);

	/**
	 * Clean up routine. The handler should unregister the listener interface
	 * and dispose any reference to the chunk reader and writer it owns.
	 */
	void disposeHandler();

	/**
	 * Interrupt any pending operation involving the ChunkReader or the
	 * ChunkWriter objects passed on initialization.
	 */
	void cancelHandler();
}
