package communication.protocol;

/**
 * Defines constants used to identify protocol chunk types.
 * 
 * @author Paolo
 */
public interface Chunk
{
	// ------------------------------------------
	// Basic protocol chunks

	/**
	 * May be sent to check connection status. It is a no-operation.
	 */
	final static int PING = 1;

	/**
	 * Response to check connection status. It is a no-operation.
	 */
	final static int PONG = 2;

	/**
	 * Use to set/get the capability mask of the end-point.
	 */
	final static int SETUP = 3;

	/**
	 * Signal the other party to disconnect.
	 */
	final static int DISCONNECT = 4;

	// ------------------------------------------
	// Video Capability

	/**
	 * Use to set/get the video stream format and chunk size.
	 */
	final static int VIDEO_CONFIG = 10;

	/**
	 * Use to start/acknowledge video streaming.
	 */
	final static int VIDEO_START = 11;

	/**
	 * Use to stop/acknowledge video streaming.
	 */
	final static int VIDEO_STOP = 12;

	/**
	 * Holds a video block as configured.
	 */
	final static int VIDEO_BLOCK = 13;

	// ------------------------------------------
	// Internal Capability

	/**
	 * Request a 3-byte write to internal registers
	 */
	final static int INTERNAL_WRITE3 = 17;

	/**
	 * Request a 2-byte write to internal registers
	 */
	final static int INTERNAL_WRITE2 = 18;

	/**
	 * Request a 1-byte write to internal registers
	 */
	final static int INTERNAL_WRITE1 = 19;

	/**
	 * Respond with error to a write request
	 */
	final static int INTERNAL_WRITEERR = 20;

	/**
	 * Respond with success to a write request
	 */
	final static int INTERNAL_WRITEOK = 21;

	/**
	 * Request a 3-byte read from internal registers
	 */
	final static int INTERNAL_READ3 = 27;

	/**
	 * Request a 2-byte read from internal registers
	 */
	final static int INTERNAL_READ2 = 28;

	/**
	 * Request a 1-byte read from internal registers
	 */
	final static int INTERNAL_READ1 = 29;

	/**
	 * Respond with error to a read request
	 */
	final static int INTERNAL_READERR = 30;

	/**
	 * Provide data of a 3-byte read request
	 */
	final static int INTERNAL_DATA1 = 31;

	/**
	 * Provide data of a 2-byte read request
	 */
	final static int INTERNAL_DATA2 = 32;

	/**
	 * Provide data of a 1-byte read request
	 */
	final static int INTERNAL_DATA3 = 33;

	// ------------------------------------------
	// Bus Capability

	/**
	 * Request and queue a generic I2C bus transfer
	 */
	final static int BUS_TRANSFER = 35;

	/**
	 * Respond and provide data to a bus transfer
	 */
	final static int BUS_RESULT = 36;

	/**
	 * Request a pause in the bus transfer queue
	 */
	final static int BUS_PAUSE = 37;

	// ------------------------------------------
	// Transfer Capability

	/**
	 * Use to open a file on the remote side
	 */
	final static int FILE_OPEN = 40;

	/**
	 * Use to close a file on the remote side
	 */
	final static int FILE_CLOSE = 41;

	/**
	 * Use to provide result of a file operation
	 */
	final static int FILE_RESULT = 42;

	/**
	 * Use to write data to a remote file
	 */
	final static int FILE_WRITE = 43;

	/**
	 * Use to read data from a remote file
	 */
	final static int FILE_READ = 44;

	/**
	 * Use to provide data of a read operation
	 */
	final static int FILE_DATA = 45;
		
	/**
	 * Request the next entry in a directory
	 */
	final static int DIR_NEXT = 46;
	
	/**
	 * Provide the entry in a directory
	 */
	final static int DIR_ENTRY = 47;
	
	/**
	 * Use to flush file buffers on disk 
	 */
	final static int FILE_FLUSH = 48;
}
