package communication.protocol;

/**
 * Interface for implementing Internal protocol capability
 * 
 * @author Paolo
 */
public interface InternalListener
{
	/**
	 * An internal read request completed with error
	 * 
	 * @see Chunk#INTERNAL_READERR
	 */
	public void onInternalReadError();

	/**
	 * An internal write request completed with error
	 * 
	 * @see Chunk#INTERNAL_WRITEERR
	 */
	public void onInternalWriteError();

	/**
	 * An internal write request completed successfully
	 * 
	 * @see Chunk#INTERNAL_WRITEOK
	 */
	public void onInternalWriteOk();

	/**
	 * Returns data of a successful internal read request
	 * 
	 * @param value
	 *            Data in the specified register
	 * 
	 * @see Chunk#INTERNAL_DATA1
	 */
	public void onInternalRegData1(int value);

	/**
	 * Returns data of a successful internal read request
	 * 
	 * @param value1
	 *            Data in the first register
	 * @param value2
	 *            Data in the second register (or the MSB in a 16-bit register)
	 * 
	 * @see Chunk#INTERNAL_DATA2
	 */
	public void onInternalRegData2(int value1, int value2);

	/**
	 * Returns data of a successful internal read request
	 * 
	 * @param value1
	 *            Data in the first register
	 * @param value2
	 *            Data in the second register (or the MSB in a 16-bit register)
	 * @param value3
	 *            Data in the third register (or the following byte in previous
	 *            registers)
	 * 
	 * @see Chunk#INTERNAL_DATA3
	 */
	public void onInternalRegData3(int value1, int value2, int value3);
}
