package communication.protocol;

/**
 * Interface for implementing basic protocol capability
 * 
 * @author Paolo
 */
public interface ProtocolListener
{
	/**
	 * Signals arrival of a new chunk (for statistical purpose or connection
	 * keep-alive)
	 * 
	 * @param type
	 *            The identifier of the chunk type
	 */
	public void onChunk(int type);

	/**
	 * Receives the reply to a ping chunk
	 * 
	 * @see Chunk#PONG
	 */
	public void onPong();

	/**
	 * Receives a remote ping request
	 * 
	 * @see Chunk#PING
	 */
	public void onPing();

	/**
	 * Receives confirmation of communication setup and approved settings.
	 * 
	 * @param capabilityMask
	 *            Supported capabilities, a combination of values defined in
	 *            {@link Capability}
	 * @param maxSpeed
	 *            Maximum bitrate at UART level (mainly statistical)
	 * 
	 * @see Chunk#SETUP
	 */
	public void onSetup(int capabilityMask, int maxSpeed);
	
	/**
	 * Receives a remote disconnection indication.
	 */
	public void onDisconnect();
}
