package communication.protocol;

/**
 * Interface for implementing Transfer protocol capability
 * 
 * @author Paolo
 */
public interface TransferListener
{
	// TODO server interface (Object Push)

	public void onTransferOpen(int flags, String name);

	public void onTransferClose(int token);

	public void onTransferWrite(int token, byte[] data);

	public void onTransferRead(int token, int size);

	public void onDirectoryNext();

	// client interface

	/**
	 * Receives the result of a file operation.
	 * 
	 * @param token
	 *            The identifier of a remote file this packet refers to
	 * @param code
	 *            The operation result identifier
	 * 
	 * @see Chunk#FILE_RESULT
	 */
	public void onTransferResult(int token, int code);

	/**
	 * Receives the actual data of a file read operation.
	 * 
	 * @param token
	 *            The identifier of a remote file this packet refers to
	 * @param data
	 *            The actual data read
	 * 
	 * @see Chunk#FILE_DATA
	 */
	public void onTransferData(int token, byte[] data);

	/**
	 * Receives an entry of a directory enumeration.
	 * 
	 * @param token
	 *            The identifier of a remote directory this packet refers to
	 * @param name
	 *            The name of the directory entry
	 * @param type
	 *            The type of the directory entry
	 * 
	 * @see Chunk#DIR_ENTRY
	 */
	public void onDirectoryEntry(int token, String name, int type);
}
