package communication.protocol;

/**
 * Interface for implementing Video protocol capability.
 * 
 * @author Paolo
 */
public interface VideoListener
{
	/**
	 * Receives the approved video configuration settings.
	 * 
	 * @param frameWidth
	 *            The video frame width. Frame height can be obtained by aspect
	 *            ratio of 4:3
	 * @param pixelFormat
	 *            The identifier of the pixel format
	 * @param frameRate
	 *            (Reserved) Suggested maximum frame rate
	 * 
	 * @see Chunk#VIDEO_CONFIG
	 */
	public void onVideoConfig(int frameWidth, int pixelFormat, int frameRate);

	/**
	 * Receives confirmation of video streaming start and the approved video
	 * block size.
	 * 
	 * @param bufferLength
	 *            The size of video blocks that will follow
	 * 
	 * @see Chunk#VIDEO_START
	 */
	public void onVideoStart(int bufferLength);

	/**
	 * Receives confirmation of video streaming end. No more video blocks can
	 * follow.
	 * 
	 * @see Chunk#VIDEO_STOP
	 */
	public void onVideoStop();

	/**
	 * Acquires a buffer to hold an incoming video block.
	 * 
	 * @return A properly sized byte array, as specified in the
	 *         {@link #onVideoStart} callback method.
	 * 
	 * @see Chunk#VIDEO_BLOCK
	 */
	public byte[] onVideoGetBlock();

	/**
	 * Releases a previously acquired buffer, filled in with video data.
	 * 
	 * @param buffer
	 *            The byte array holding a new video block.
	 * 
	 * @see Chunk#VIDEO_BLOCK
	 */
	public void onVideoSetBlock(byte[] buffer);
}
