/**
 * 
 */
package communication.transport;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Interface for connection providers implementations.
 * 
 * Used to abstract from the physical communication media or application
 * protocol encapsulation. The communication uses standard InputStream and
 * OuputStream interfaces.
 * 
 * @author Paolo
 */
public interface ConnectionProvider
{
	/**
	 * Set if InputStream.available() can be used to determine
	 * InputStream.read() blocking behavior.
	 */
	public static final int AVAILABLE_IMPLEMENTED = 1;

	/**
	 * Set if InputStream.read() does not block when no further bytes are
	 * available.
	 */
	public static final int READ_NON_BLOCKING = 2;

	/**
	 * Set if InputStream.read() does not return when Thread.interrupt() is
	 * called on the reading thread.
	 */
	public static final int READ_NON_INTERRUPTIBLE = 4;

	/**
	 * @return The input/output stream properties
	 * 
	 * @see #AVAILABLE_IMPLEMENTED
	 * @see #READ_NON_BLOCKING
	 * @see #READ_NON_INTERRUPTIBLE
	 */
	public int getStreamProperties();

	/**
	 * @return The input stream
	 */
	public InputStream getInputStream();

	/**
	 * @return The output stream
	 */
	public OutputStream getOutputStream();

	/**
	 * The input and output streams are valid from now on.
	 * 
	 * @throws IOException
	 */
	public boolean openConnection() throws IOException;

	/**
	 * The input and output streams cease to be valid afterwards.
	 * 
	 * @throws IOException
	 */
	public void closeConnection() throws IOException;
}
